@@ -18,7 +18,7 @@ use Carp;
 use strict;
 
 @Chart::Composite::ISA = qw(Chart::Base);
-$Chart::Composite::VERSION = 0.99;
+$Chart::Composite::VERSION = '2.0';
 
 #>>>>>>>>>>>>>>>>>>>>>>>>>>#
 #  public methods go here  #
@@ -62,7 +62,7 @@ sub imagemap_dump {
   # croak if they didn't ask me to remember the data, or if they're asking
   # for the data before I generate it
   unless (($self->{'imagemap'} =~ /^true$/i) && $self->{'imagemap_data'}) {
-    croak "You need to set the imagemap option to true, and then call the gif method, before you can get the imagemap data";
+    croak "You need to set the imagemap option to true, and then call the png method, before you can get the imagemap data";
   }
 
   # make a copy for them, and reorder it
@@ -140,10 +140,11 @@ sub _split_data {
   my @types = ($self->{'composite_info'}[0][0],$self->{'composite_info'}[1][0]);
   my ($ref, $i, $j);
 
-  # we can only do two at a time
-  if ($self->{'composite_info'}[2]) {
-    croak "Sorry, Chart::Composite can only do two chart types at a time";
-  }
+## Already checked for number of components in _check_data, above.
+#   # we can only do two at a time
+#   if ($self->{'composite_info'}[2]) {
+#     croak "Sorry, Chart::Composite can only do two chart types at a time";
+#   }
 
   # load the individual modules
   require "Chart/".$types[0].".pm";
@@ -192,6 +193,7 @@ sub _split_data {
     $self->{'component_datasets'}[$i] = $self->{'composite_info'}[$i][1];
     push @{$ref}, $self->{'dataref'}[0];
     for $j (@{$self->{'composite_info'}[$i][1]}) {
+      $self->_color_role_to_index('dataset'.($j-1)); # allocate color index
       push @{$ref}, $self->{'dataref'}[$j];
     }
     $self->{'sub_'.$i}->_copy_data ($ref);
@@ -229,6 +231,54 @@ sub _split_data {
   return;
 }
 
+sub _draw_legend {
+  my $self = shift;
+  my ($length);
+ 
+  # check to see if they have as many labels as datasets,
+  # warn them if not
+  if (($#{$self->{'legend_labels'}} >= 0) &&
+       ((scalar(@{$self->{'legend_labels'}})) != $self->{'num_datasets'})) {
+    carp "The number of legend labels and datasets doesn\'t match";
+  }
+ 
+  # init a field to store the length of the longest legend label
+  unless ($self->{'max_legend_label'}) {
+    $self->{'max_legend_label'} = 0;
+  }
+ 
+  # fill in the legend labels, find the longest one
+  for (1..$self->{'num_datasets'}) {
+    unless ($self->{'legend_labels'}[$_-1]) {
+      $self->{'legend_labels'}[$_-1] = "Dataset $_";
+    }
+    $length = length($self->{'legend_labels'}[$_-1]);
+    if ($length > $self->{'max_legend_label'}) {
+      $self->{'max_legend_label'} = $length;
+    }
+  }
+ 
+  # different legend types
+  if ($self->{'legend'} eq 'bottom') {
+    $self->_draw_bottom_legend;
+  }
+  elsif ($self->{'legend'} eq 'right') {
+    $self->_draw_right_legend;
+  }
+  elsif ($self->{'legend'} eq 'left') {
+    $self->_draw_left_legend;
+  }
+  elsif ($self->{'legend'} eq 'top') {
+    $self->_draw_top_legend;
+  } elsif ($self->{'legend'} eq 'none') {
+    $self->_draw_none_legend;
+  } else {
+    carp "I can't put a legend there\n";
+  }
+ 
+  # and return
+  return 1;
+}
 
 ## put the legend on the top of the data plot
 sub _draw_top_legend {
@@ -237,17 +287,39 @@ sub _draw_top_legend {
   my ($x1, $y1, $x2, $y2, $empty_width, $max_label_width, $cols, $rows, $color);
   my ($col_width, $row_height, $i, $j, $r, $c, $index, $x, $y, $sub, $w, $h);
   my $font = $self->{'legend_font'};
-  my @colors = qw(light_purple pink peach olive plum turquoise mauve brown);
+  my (%colors, @datasets);
 
   # copy the current boundaries into the sub-objects
   $self->_sub_update;
 
+## Make datasetI numbers match indexes of @{ $self->{'dataref'} }[1.....].
+#   # modify the dataset color table entries to avoid duplicating
+#   # dataset colors (this limits the number of possible data sets
+#   # for each component to 8)
+#   for (0..7) {
+#     $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
+#       = $self->{'color_table'}{'dataset'.($_+8)};
+#   }
   # modify the dataset color table entries to avoid duplicating
-  # dataset colors (this limits the number of possible data sets
-  # for each component to 8)
-  for (0..$#colors) {
+  # dataset colors.
+  my ($n0, $n1) = map { scalar @{ $self->{'composite_info'}[$_][1] } } 0..1;
+  for (0..$n1-1) {
     $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
-      = $self->{'color_table'}{$colors[$_]};
+      = $self->{'color_table'}{'dataset'.($_+$n0)};
+  }
+
+  # make sure we use the right colors for the legend
+  @datasets = @{$self->{'composite_info'}[0][1]};
+  $i = 0;
+  for (0..$#datasets) {
+    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i)};
+    $i++;
+  }
+  @datasets = @{$self->{'composite_info'}[1][1]};
+  $i = 0;
+  for (0..$#datasets) {
+    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i+$n0)};
+    $i++;
   }
 
   # make sure we're using a real font
@@ -299,7 +371,7 @@ sub _draw_top_legend {
   $y2 = $self->{'curr_y_min'} + $self->{'text_space'}
           + ($rows * $row_height) + (2 * $self->{'legend_space'});
   $self->{'gd_obj'}->rectangle($x1, $y1, $x2, $y2, 
-                               $self->{'color_table'}{'misc'});
+                               $self->_color_role_to_index('misc'));
 
   # leave some space inside the legend
   $x1 += $self->{'legend_space'} + $self->{'text_space'};
@@ -362,12 +434,19 @@ sub _draw_right_legend {
   # copy the current boundaries and colors into the sub-objects
   $self->_sub_update;
 
+#   # modify the dataset color table entries to avoid duplicating
+#   # dataset colors (this limits the number of possible data sets
+#   # for each component to 8)
+#   for (0..7) {
+#     $self->{'sub_1'}{'color_table'}{'dataset'.$_}
+#       = $self->{'color_table'}{'dataset'.($_+8)};
+#   }
   # modify the dataset color table entries to avoid duplicating
-  # dataset colors (this limits the number of possible data sets
-  # for each component to 8)
-  for (0..7) {
-    $self->{'sub_1'}{'color_table'}{'dataset'.$_}
-      = $self->{'color_table'}{'dataset'.($_+8)};
+  # dataset colors.
+  my ($n0, $n1) = map { scalar @{ $self->{'composite_info'}[$_][1] } } 0..1;
+  for (0..$n1-1) {
+    $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
+      = $self->{'color_table'}{'dataset'.($_+$n0)};
   }
 
   # make sure we use the right colors for the legend
@@ -380,7 +459,7 @@ sub _draw_right_legend {
   @datasets = @{$self->{'composite_info'}[1][1]};
   $i = 0;
   for (0..$#datasets) {
-    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i+8)};
+    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i+$n0)};
     $i++;
   }
 
@@ -393,7 +472,7 @@ sub _draw_right_legend {
   ($h, $w) = ($font->height, $font->width);
 
   # get the miscellaneous color
-  $misccolor = $self->{'color_table'}{'misc'};
+  $misccolor = $self->_color_role_to_index('misc');
 
   # find out how wide the largest label is
   $width = (2 * $self->{'text_space'})
@@ -440,6 +519,105 @@ sub _draw_right_legend {
   return;
 }
 
+##  draw the legend at the left of the data plot
+sub _draw_left_legend {
+  my $self = shift;
+  my @labels = @{$self->{'legend_labels'}};
+  my ($x1, $x2, $x3, $y1, $y2, $width, $color, $misccolor, $w, $h);
+  my $font = $self->{'legend_font'};
+  my (%colors, @datasets, $i);
+ 
+  # copy the current boundaries and colors into the sub-objects
+  $self->_sub_update;
+ 
+#   # modify the dataset color table entries to avoid duplicating
+#   # dataset colors (this limits the number of possible data sets
+#   # for each component to 8)
+#   for (0..7) {
+#     $self->{'sub_1'}{'color_table'}{'dataset'.$_}
+#       = $self->{'color_table'}{'dataset'.($_+8)};
+#   }
+  # modify the dataset color table entries to avoid duplicating
+  # dataset colors.
+  my ($n0, $n1) = map { scalar @{ $self->{'composite_info'}[$_][1] } } 0..1;
+  for (0..$n1-1) {
+    $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
+      = $self->{'color_table'}{'dataset'.($_+$n0)};
+  }
+
+  # make sure we use the right colors for the legend
+  @datasets = @{$self->{'composite_info'}[0][1]};
+  $i = 0;
+  for (0..$#datasets) {
+    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i)};
+    $i++;
+  }
+  @datasets = @{$self->{'composite_info'}[1][1]};
+  $i = 0;
+  for (0..$#datasets) {
+    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i+$n0)};
+    $i++;
+  }
+ 
+  # make sure we're using a real font
+  unless ((ref ($font)) eq 'GD::Font') {
+    croak "The subtitle font you specified isn\'t a GD Font object";
+  }
+ 
+  # get the size of the font
+  ($h, $w) = ($font->height, $font->width);
+ 
+  # get the miscellaneous color
+  $misccolor = $self->_color_role_to_index('misc');
+ 
+  # find out how wide the largest label is
+  $width = (2 * $self->{'text_space'})
+    + ($self->{'max_legend_label'} * $w)
+    + $self->{'legend_example_size'}
+    + (2 * $self->{'legend_space'});
+ 
+  # get some base x-y coordinates
+  $x1 = $self->{'curr_x_min'};
+  $x2 = $self->{'curr_x_min'} + $width;
+  $y1 = $self->{'curr_y_min'} + $self->{'graph_border'} ;
+  $y2 = $self->{'curr_y_min'} + $self->{'graph_border'} + $self->{'text_space'
+}
+          + ($self->{'num_datasets'} * ($h + $self->{'text_space'}))
+          + (2 * $self->{'legend_space'});
+ 
+  # box the legend off
+  $self->{'gd_obj'}->rectangle ($x1, $y1, $x2, $y2, $misccolor);
+ 
+  # leave that nice space inside the legend box
+  $x1 += $self->{'legend_space'};
+  $y1 += $self->{'legend_space'} + $self->{'text_space'};
+ 
+  # now draw the actual legend
+  for (0..$#labels) {
+    # get the color
+    $color = $colors{$_};
+ 
+    # find the x-y coords
+    $x2 = $x1;
+    $x3 = $x2 + $self->{'legend_example_size'};
+    $y2 = $y1 + ($_ * ($self->{'text_space'} + $h)) + $h/2;
+ 
+    # do the line first
+    $self->{'gd_obj'}->line ($x2, $y2, $x3, $y2, $color);
+ 
+    # now the label
+    $x2 = $x3 + (2 * $self->{'text_space'});
+    $y2 -= $h/2;
+    $self->{'gd_obj'}->string ($font, $x2, $y2, $labels[$_], $color);
+  }
+ 
+  # mark off the used space
+  $self->{'curr_x_min'} += $width;
+ 
+  # and return
+  return 1;
+}
+
 
 ##  draw the legend on the bottom of the data plot
 sub _draw_bottom_legend {
@@ -448,17 +626,37 @@ sub _draw_bottom_legend {
   my ($x1, $y1, $x2, $y2, $empty_width, $max_label_width, $cols, $rows, $color);
   my ($col_width, $row_height, $i, $j, $r, $c, $index, $x, $y, $sub, $w, $h);
   my $font = $self->{'legend_font'};
-  my @colors = qw(light_purple pink peach olive plum turquoise mauve brown);
+  my (%colors, @datasets);
 
   # copy the current boundaries and colors into the sub-objects
   $self->_sub_update;
 
+#   # modify the dataset color table entries to avoid duplicating
+#   # dataset colors (this limits the number of possible data sets
+#   # for each component to 8)
+#   for (0..7) {
+#     $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
+#       = $self->{'color_table'}{'dataset'.($_+8)};
+#   }
   # modify the dataset color table entries to avoid duplicating
-  # dataset colors (this limits the number of possible data sets
-  # for each component to 8)
-  for (0..$#colors) {
+  # dataset colors.
+  my ($n0, $n1) = map { scalar @{ $self->{'composite_info'}[$_][1] } } 0..1;
+  for (0..$n1-1) {
     $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
-      = $self->{'color_table'}{$colors[$_]};
+      = $self->{'color_table'}{'dataset'.($_+$n0)};
+  }
+
+  @datasets = @{$self->{'composite_info'}[0][1]};
+  $i = 0;
+  for (0..$#datasets) {
+    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i)};
+    $i++;
+  }
+  @datasets = @{$self->{'composite_info'}[1][1]};
+  $i = 0;
+  for (0..$#datasets) {
+    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i+$n0)};
+    $i++;
   }
 
   # make sure we're using a real font
@@ -507,7 +705,7 @@ sub _draw_bottom_legend {
           - ($rows * $row_height) - (2 * $self->{'legend_space'});
   $y2 = $self->{'curr_y_max'};
   $self->{'gd_obj'}->rectangle($x1, $y1, $x2, $y2, 
-                               $self->{'color_table'}{'misc'});
+                               $self->_color_role_to_index('misc'));
   $x1 += $self->{'legend_space'} + $self->{'text_space'};
   $x2 -= $self->{'legend_space'};
   $y1 += $self->{'legend_space'} + $self->{'text_space'};
@@ -549,6 +747,25 @@ sub _draw_bottom_legend {
   return;
 }
 
+# no legend to draw.. just update the color tables for subs
+sub _draw_none_legend {
+  my $self = shift;
+
+  $self->_sub_update();
+
+#   for (0..7) {
+#     $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
+#        = $self->{'color_table'}{'dataset'.($_+8)};
+#    }
+  # modify the dataset color table entries to avoid duplicating
+  # dataset colors.
+  my ($n0, $n1) = map { scalar @{ $self->{'composite_info'}[$_][1] } } 0..1;
+  for (0..$n1-1) {
+    $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
+      = $self->{'color_table'}{'dataset'.($_+$n0)};
+  }
+}
+
 
 ## draw the ticks and tick labels
 sub _draw_ticks {
@@ -574,8 +791,8 @@ sub _draw_x_ticks {
   my $self = shift;
   my $data = $self->{'dataref'};
   my $font = $self->{'tick_label_font'};
-  my $textcolor = $self->{'color_table'}{'text'};
-  my $misccolor = $self->{'color_table'}{'misc'};
+  my $textcolor = $self->_color_role_to_index('text');
+  my $misccolor = $self->_color_role_to_index('misc');
   my ($h, $w);
   my ($x1, $x2, $y1, $y2);
   my ($width, $delta);
@@ -593,10 +810,16 @@ sub _draw_x_ticks {
 
   # allow for the amount of space the y-ticks will push the
   # axes over to the right and to the left
+## _draw_y_ticks allows 3 * text_space, not 2 * ;  this caused mismatch between
+## the ticks (and grid lines) and the data.
+#   $x1 = $self->{'curr_x_min'} + ($w * $self->{'y_tick_label_length1'})
+#          + (2 * $self->{'text_space'}) + $self->{'tick_len'};
+#   $x2 = $self->{'curr_x_max'} - ($w * $self->{'y_tick_label_length2'})
+#          - (2 * $self->{'text_space'}) - $self->{'tick_len'};
   $x1 = $self->{'curr_x_min'} + ($w * $self->{'y_tick_label_length1'})
-         + (2 * $self->{'text_space'}) + $self->{'tick_len'};
+         + (3 * $self->{'text_space'}) + $self->{'tick_len'};
   $x2 = $self->{'curr_x_max'} - ($w * $self->{'y_tick_label_length2'})
-         - (2 * $self->{'text_space'}) - $self->{'tick_len'};
+         - (3 * $self->{'text_space'}) - $self->{'tick_len'};
   $y1 = $self->{'curr_y_max'} - $h - $self->{'text_space'};
 
   # get the delta value, figure out how to draw the labels
@@ -613,22 +836,23 @@ sub _draw_x_ticks {
     if ($self->{'skip_x_ticks'}) {
       for (0..int(($self->{'num_datapoints'}-1)/$self->{'skip_x_ticks'})) {
         $x2 = $x1 + ($delta/2) + ($delta*($_*$self->{'skip_x_ticks'})) 
-	      - ($w*length($data->[0][$_* $self->{'skip_x_ticks'}])) / 2; 
+	      - ($w*length( $self->{'f_x_tick'}->($data->[0][$_* $self->{'skip_x_ticks'}]))) / 2;
         $self->{'gd_obj'}->string($font, $x2, $y1, 
-	                          $data->[0][$_*$self->{'skip_x_ticks'}], 
+	                          $self->{'f_x_tick'}->($data->[0][$_*$self->{'skip_x_ticks'}]),
 				  $textcolor);
       }
     }
     elsif ($self->{'custom_x_ticks'}) {
       for (@{$self->{'custom_x_ticks'}}) {
-        $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length($data->[0][$_])) / 2; 
-        $self->{'gd_obj'}->string($font, $x2, $y1, $data->[0][$_], $textcolor);
+        $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length( $self->{'f_x_tick'}->($data->[0][$_]))) / 2;
+        $self->{'gd_obj'}->string($font, $x2, $y1,
+                                  $self->{'f_x_tick'}->($data->[0][$_]), $textcolor);
       }
     }
     else {
       for (0..$self->{'num_datapoints'}-1) {
-        $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length($data->[0][$_])) / 2; 
-        $self->{'gd_obj'}->string($font, $x2, $y1, $data->[0][$_], $textcolor);
+        $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length($self->{'f_x_tick'}->($data->[0][$_]))) / 2;
+        $self->{'gd_obj'}->string($font, $x2, $y1, $self->{'f_x_tick'}->($data->[0][$_]), $textcolor);
       }
     }
   }
@@ -637,12 +861,12 @@ sub _draw_x_ticks {
       $stag = 0;
       for (0..int(($self->{'num_datapoints'}-1)/$self->{'skip_x_ticks'})) {
         $x2 = $x1 + ($delta/2) + ($delta*($_*$self->{'skip_x_ticks'})) 
-	        - ($w*length($data->[0][$_*$self->{'skip_x_ticks'}])) / 2;
+	        - ($w*length( $self->{'f_x_tick'}->($data->[0][$_*$self->{'skip_x_ticks'}]))) / 2;
         if (($stag % 2) == 1) {
           $y1 -= $self->{'text_space'} + $h;
         }
         $self->{'gd_obj'}->string($font, $x2, $y1, 
-                                  $data->[0][$_*$self->{'skip_x_ticks'}], 
+                                  $self->{'f_x_tick'}->($data->[0][$_*$self->{'skip_x_ticks'}]),
                                   $textcolor);
         if (($stag % 2) == 1) {
           $y1 += $self->{'text_space'} + $h;
@@ -653,11 +877,11 @@ sub _draw_x_ticks {
     elsif ($self->{'custom_x_ticks'}) {
       $stag = 0;
       for (sort (@{$self->{'custom_x_ticks'}})) {
-        $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length($data->[0][$_])) / 2;
+        $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length( $self->{'f_x_tick'}->($data->[0][$_]))) / 2;
         if (($stag % 2) == 1) {
           $y1 -= $self->{'text_space'} + $h;
         }
-        $self->{'gd_obj'}->string($font, $x2, $y1, $data->[0][$_], $textcolor);
+        $self->{'gd_obj'}->string($font, $x2, $y1,  $self->{'f_x_tick'}->($data->[0][$_]), $textcolor);
         if (($stag % 2) == 1) {
           $y1 += $self->{'text_space'} + $h;
         }
@@ -666,11 +890,11 @@ sub _draw_x_ticks {
     }
     else {
       for (0..$self->{'num_datapoints'}-1) {
-        $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length($data->[0][$_])) / 2;
+        $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length( $self->{'f_x_tick'}->($data->[0][$_]))) / 2;
         if (($_ % 2) == 1) {
           $y1 -= $self->{'text_space'} + $h;
         }
-        $self->{'gd_obj'}->string($font, $x2, $y1, $data->[0][$_], $textcolor);
+        $self->{'gd_obj'}->string($font, $x2, $y1,  $self->{'f_x_tick'}->($data->[0][$_]), $textcolor);
         if (($_ % 2) == 1) {
           $y1 += $self->{'text_space'} + $h;
         }
@@ -683,28 +907,28 @@ sub _draw_x_ticks {
       for (0..int(($self->{'num_datapoints'}-1)/$self->{'skip_x_ticks'})) {
         $x2 = $x1 + ($delta/2) + ($delta*($_*$self->{'skip_x_ticks'})) - $h/2;
         $y2 = $y1 - (($self->{'x_tick_label_length'} 
-	              - length($data->[0][$_*$self->{'skip_x_ticks'}])) * $w);
+	              - length( $self->{'f_x_tick'}->($data->[0][$_*$self->{'skip_x_ticks'}]))) * $w);
         $self->{'gd_obj'}->stringUp($font, $x2, $y2, 
-	                            $data->[0][$_*$self->{'skip_x_ticks'}], 
+                                    $self->{'f_x_tick'}->($data->[0][$_*$self->{'skip_x_ticks'}]),
 				    $textcolor);
       }
     }
     elsif ($self->{'custom_x_ticks'}) {
       for (@{$self->{'custom_x_ticks'}}) {
         $x2 = $x1 + ($delta/2) + ($delta*$_) - $h/2;
-        $y2 = $y1 - (($self->{'x_tick_label_length'} - length($data->[0][$_])) 
+        $y2 = $y1 - (($self->{'x_tick_label_length'} - length( $self->{'f_x_tick'}->($data->[0][$_])))
                       * $w);
         $self->{'gd_obj'}->stringUp($font, $x2, $y2, 
-	                            $data->[0][$_], $textcolor);
+                                    $self->{'f_x_tick'}->($data->[0][$_]), $textcolor);
       }
     }
     else {
       for (0..$self->{'num_datapoints'}-1) {
         $x2 = $x1 + ($delta/2) + ($delta*$_) - $h/2;
-        $y2 = $y1 - (($self->{'x_tick_label_length'} - length($data->[0][$_])) 
+        $y2 = $y1 - (($self->{'x_tick_label_length'} - length( $self->{'f_x_tick'}->($data->[0][$_])))
                       * $w);
         $self->{'gd_obj'}->stringUp($font, $x2, $y2, 
-	                            $data->[0][$_], $textcolor);
+	                             $self->{'f_x_tick'}->($data->[0][$_]), $textcolor);
       }
     }
   }
@@ -812,6 +1036,12 @@ sub _draw_data {
   # do a final bounds update
   $self->_boundary_update ($self, $self->{'sub_0'});
   $self->_boundary_update ($self, $self->{'sub_1'});
+  
+
+  # init the imagemap data field if they wanted it
+  if ($self->{'imagemap'} =~ /^true$/i) {
+    $self->{'imagemap_data'} = [];
+  }
 
   # now let the component modules go to work
   $self->{'sub_0'}->_draw_data;
